﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Sokit
{
    public partial class FormMain : Form
    {
        public FormMain()
        {
            InitializeComponent();
        }
        private void FormMain_Load(object sender, EventArgs e)
        {
            cb_TransferType.SelectedIndex = 0;
        }
        // 全局UDP的服务器监听
        private UdpClient UDPTransfer_Listener = null;
        // 转发者
        private Dictionary<string, TransferModel> Dic_Transfer = new Dictionary<string, TransferModel>();

        private static readonly object Lock = new object();

        private void btn_Transfer_Start_Click(object sender, EventArgs e)
        {
            string Transfer_Type = cb_TransferType.Text;
            if (Transfer_Type.Equals("UDP"))
            {
                if (btn_Transfer_Start.Text.Equals("Start"))
                {
                    bool flag = StartUDPTransfer();
                    if (flag)
                    {
                        this.btn_Transfer_Start.Text = "Stop";
                        this.btn_Transfer_Start.BackColor = Color.LightGray;
                    }

                }
                else if (btn_Transfer_Start.Text.Equals("Stop"))
                {
                    StopUDPTransfer();
                    this.btn_Transfer_Start.Text = "Start";
                    this.btn_Transfer_Start.BackColor = Color.Transparent;
                }

            }
        }

        // 启动UDP转发功能
        private bool StartUDPTransfer()
        {
            bool ret = false;

            string srcip = cb_srcIP.Text;
            string srcport = cb_srcport.Text;
            string dstip = cb_dstIP.Text;
            string dstport = cb_dstport.Text;

            // 本地监听的IP
            IPAddress SRC_IP = ((srcip.Equals("本地  [*]"))) ? (IPAddress.Any) : (IPAddress.Parse(srcip));
            // 本地监听的端口
            int SRC_Port = int.Parse(srcport);

            try
            {
                UDPTransfer_Listener = new UdpClient(new IPEndPoint(SRC_IP, SRC_Port));
                this.Invoke(new MethodInvoker(() =>
                {
                    TreeNode tn = this.treeView_log.Nodes.Add(DateTime.Now.ToString("HH:mm:ss") + " 转发服务开启：成功");
                }));
                ret = true;
            }
            catch (Exception ex)
            {
                this.Invoke(new MethodInvoker(() =>
                {
                    TreeNode tn = this.treeView_log.Nodes.Add(DateTime.Now.ToString("HH:mm:ss") + " 转发服务开启：失败");
                }));
                return false;
            }


            Task.Factory.StartNew(() =>
            {
                while (UDPTransfer_Listener != null)
                {
                    try
                    {

                        IPEndPoint remoteEP = new IPEndPoint(IPAddress.Any, 0);
                        byte[] receive_byte_array = UDPTransfer_Listener.Receive(ref remoteEP);

                        LogMSGToDebug(receive_byte_array.ToString());
                        this.Invoke(new MethodInvoker(() =>
                        {
                            TreeNode tn = this.treeView_log.Nodes.Add(DateTime.Now.ToString("HH:mm:ss") + " - ->> " + String.Format("{0}:{1}", remoteEP.Address, remoteEP.Port));
                            tn.Nodes.Add(BitConverter.ToString(receive_byte_array, 0).ToUpper());
                            lbl_receive.Text = (int.Parse(lbl_receive.Text) + receive_byte_array.Length).ToString();
                        }));

                        // 目标IP
                        IPAddress DST_IP = IPAddress.Parse(dstip);
                        // 目标端口
                        int DST_Port = int.Parse(dstport);


                        int TransferLocalSendPort = 0;
                        if (!Dic_Transfer.ContainsKey(string.Format("{0}:{1}", remoteEP.Address, remoteEP.Port)))
                        {
                            TransferLocalSendPort = remoteEP.Port;

                            while (PortHelper.PortInUse(TransferLocalSendPort, PortType.UDP))
                            {

                                if (TransferLocalSendPort >= 65535)
                                {
                                    TransferLocalSendPort--;
                                }
                                else
                                {
                                    TransferLocalSendPort++;
                                }
                            }

                            TransferModel tmodel = new TransferModel()
                            {
                                Client_IEP = remoteEP,
                                LocalSendClient = new UdpClient(new IPEndPoint(IPAddress.Any, TransferLocalSendPort)),
                                DST_IEP = new IPEndPoint(DST_IP, DST_Port),
                                LocalSendClient_Received = false,
                                UpdateDt=DateTime.Now
                            };
                            Dic_Transfer.Add(string.Format("{0}:{1}", remoteEP.Address, remoteEP.Port), tmodel);
                            this.Invoke(new MethodInvoker(() =>
                            {
                                listBox_conn.Items.Add(string.Format("{0}:{1}", remoteEP.Address, remoteEP.Port));
                            }));
                            
                        }
                        else
                        {
                            Dic_Transfer[string.Format("{0}:{1}", remoteEP.Address, remoteEP.Port)].UpdateDt = DateTime.Now;
                        }


                        TransferUDPData(Dic_Transfer[string.Format("{0}:{1}", remoteEP.Address, remoteEP.Port)], receive_byte_array);

                    }
                    catch (Exception ex)
                    {
                        LogMSGToDebug(ex.Message);
                    }
                }

                this.Invoke(new MethodInvoker(() =>
                {
                    TreeNode tn = this.treeView_log.Nodes.Add(DateTime.Now.ToString("HH:mm:ss") + " 服务任务关闭. . .");
                }));

            });

            return ret;
        }

        // 关闭UDP转发功能
        private void StopUDPTransfer()
        {
            if (UDPTransfer_Listener == null) { return; }

            LogMSGToDebug("连接数量：" + Dic_Transfer.Count());
            foreach (KeyValuePair<string, TransferModel> t in Dic_Transfer)
            {
                TransferModel tModel = (t.Value as TransferModel);
                tModel.LocalSendClient.Close();
            }
            Dic_Transfer.Clear();

            UDPTransfer_Listener.Close();
            UDPTransfer_Listener = null;
        }


        private void TransferUDPData(TransferModel tModel, byte[] data)
        {
            try
            {
                UdpClient sendClient = tModel.LocalSendClient;
                IPEndPoint iep = tModel.DST_IEP;

                sendClient.Send(data, data.Length, iep);
                //Byte[] receiveBytes = sendClient.Receive(ref iep);
                //CallBack_ReceiveData(receiveBytes);


                if (!tModel.LocalSendClient_Received)
                {
                    sendClient.BeginReceive(EndReceive, tModel);
                    tModel.LocalSendClient_Received = true;
                }

            }
            catch (Exception ex)
            {
                LogMSGToDebug(ex.Message);
            }

        }

        private void EndReceive(IAsyncResult ar)
        {
            try
            {
                TransferModel tModel = (ar.AsyncState as TransferModel);

                UdpClient sendClient = tModel.LocalSendClient;
                IPEndPoint iep = tModel.DST_IEP;
                IPEndPoint client_iep = tModel.Client_IEP;

                Byte[] receiveBytes = sendClient.EndReceive(ar, ref iep);

                UDPTransfer_Listener.Send(receiveBytes, receiveBytes.Length, client_iep);

                this.Invoke(new MethodInvoker(() =>
                {
                    TreeNode tn = this.treeView_log.Nodes.Add(DateTime.Now.ToString("HH:mm:ss") + " <<- - " + string.Format("{0}:{1}", client_iep.Address, client_iep.Port));
                    tn.Nodes.Add(BitConverter.ToString(receiveBytes, 0).ToLower());
                    lbl_send.Text = (int.Parse(lbl_send.Text) + receiveBytes.Length).ToString();
                }));

                sendClient.BeginReceive(EndReceive, tModel);

            }
            catch (Exception ex)
            {
                LogMSGToDebug(ex.Message);
            }
        }
        private void LogMSGToDebug(string msg)
        {
            this.Invoke(new MethodInvoker(() =>
            {
                System.Diagnostics.Trace.WriteLine(DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss") + " " + msg);
            }));
        }

        private void btn_clear_Click(object sender, EventArgs e)
        {
            treeView_log.Nodes.Clear();
            lbl_receive.Text = "0";
            lbl_send.Text = "0";
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            if (treeView_log.Nodes.Count > 1000)
            {
                treeView_log.Nodes.Clear();
                lbl_receive.Text = "0";
                lbl_send.Text = "0";
            }

            List<string> needRemove = new List<string>();
            foreach (KeyValuePair<string, TransferModel> t in Dic_Transfer)
            {
                TransferModel tModel = (t.Value as TransferModel);
                TimeSpan ts=DateTime.Now.Subtract(tModel.UpdateDt);
                if(ts.TotalSeconds > 20){
                    tModel.LocalSendClient.Close();
                    needRemove.Add(string.Format("{0}:{1}", tModel.Client_IEP.Address, tModel.Client_IEP.Port));
                }
            }

            lock (Lock)
            {
                foreach (string str in needRemove)
                {
                    Dic_Transfer.Remove(str);
                    listBox_conn.Items.Remove(str);
                }
            }

            groupBox2.Text = "当前连接（" + listBox_conn.Items.Count+" / "+Dic_Transfer.Count + "）";
        }

        private void btn_selectall_Click(object sender, EventArgs e)
        {
            
            for(int i=0;i<listBox_conn.Items.Count;i++)
            {
                listBox_conn.SetSelected(i, true);
            }
        }

        private void btn_close_Click(object sender, EventArgs e)
        {
            List<string> needRemove = new List<string>();
            for (int i = 0; i < listBox_conn.SelectedItems.Count; i++)
            {
                needRemove.Add(listBox_conn.SelectedItems[i].ToString());
            }
            lock (Lock) { 
                foreach (string str in needRemove)
                {
                    Dic_Transfer.Remove(str);
                    listBox_conn.Items.Remove(str);
                }
            }
        }


    }

    public class TransferModel
    {
        public IPEndPoint Client_IEP;
        public UdpClient LocalSendClient;
        public bool LocalSendClient_Received;
        public IPEndPoint DST_IEP;
        public DateTime UpdateDt;
    }
}
